sap.ui.define(["bchui5/mvc/controller/AbstractController", "bchui5/util/BCHLogger", "bchui5/util/Debug", "bchui5/util/auth/AuthInterfaces", "exceptiontime/util/Enums", "sap/me/CalendarEventType", "sap/me/CalendarDate"], function (__AbstractController, __BCHLogger, __Debug, __bchui5_util_auth_AuthInterfaces, __exceptiontime_util_Enums, CalendarEventType, CalendarDate) {
  function _interopRequireDefault(obj) {
    return obj && obj.__esModule && typeof obj.default !== "undefined" ? obj.default : obj;
  }

  const AbstractController = _interopRequireDefault(__AbstractController);

  const BCHLogger = _interopRequireDefault(__BCHLogger);

  const Debug = _interopRequireDefault(__Debug);

  const AuthEventIds = __bchui5_util_auth_AuthInterfaces["AuthEventIds"];
  const ModelNames = __exceptiontime_util_Enums["ModelNames"];
  const logger = new BCHLogger("Calendar.controller");
  const CalendarSelectionMode = sap.me.CalendarSelectionMode; // this can't be imported

  const Legend = {
    Existing: CalendarEventType.Type04
  };
  /**
   * @name exceptiontime.controller.Calendar
   * @controller
   */

  const CalendarController = AbstractController.extend("exceptiontime.controller.Calendar", {
    constructor: function constructor() {
      AbstractController.prototype.constructor.apply(this, arguments);
      this.calendar = this.byId("calendar");
      this.selectButton = this.byId("selectBtn");
      this.clearButton = this.byId("clearBtn");
      this.modeSelect = this.byId("modeSelect");
    },
    onInit: function _onInit() {
      this.calendar.setSelectionMode(CalendarSelectionMode.SINGLE);
      const component = this.getComponent();
      component.getAuthUtil().observeEvents().filter(event => event.name === AuthEventIds.LoggedOut).subscribe(this.handleLoggedout.bind(this));
      this.getEventBus().subscribe("odata", "entries", this.highlightExistingEntryDates, this);
      Debug.add({
        calendar: this.calendar
      });
    },
    onBeforeShow: function _onBeforeShow() {
      if (this.isSingleMode()) {
        if (this.calendar.getSelectedDates().length === 0) {
          // set the current date as the 'previously' selected. mainly just to simplify the legend
          this.calendar.toggleDatesSelection([new CalendarDate()], true);
        }
      }

      this.enableOrDisableSelect();
    },
    onBeforeFirstShow: function _onBeforeFirstShow() {
      this.highlightExistingEntryDates();
    },
    onCancelPress: function _onCancelPress() {
      // TODO in multi mode, revert to the dates that were selected when they opened the calendar.
      // Timeout for a bit to give the user a second to lift their finger before the nav-ing back (maybe just a chrome emulation issue)
      setTimeout(this.getRouter().navBack, 10);
    },
    onSelectPress: function _onSelectPress() {
      const selectedDates = this.calendar.getSelectedDates().map(date => new Date(date)).sort((a, b) => a - b);
      this.getRouter().navBack({
        data: {
          selectedDates
        }
      });
    },
    onClearPress: function _onClearPress() {
      this.calendar.unselectAllDates();
      this.enableOrDisableSelect();
    },
    onDateChange: function _onDateChange() {},
    onDateTap: function _onDateTap(evt) {
      logger.start("onDateTap");

      if (this.isSingleMode()) {
        // When the user selects the currently selected date, it actually un-selects it.
        // That's not the behaviour we want, so we need to re-select it on the calendar.
        // Then 'onSelectPress' can get it from the calendar.
        if (!evt.getParameter("didSelect")) {
          this.calendar.toggleDatesSelection([evt.getParameter("date")], true);
        }

        this.onSelectPress();
      } else {
        this.enableOrDisableSelect();
      }
    },
    onChangeRange: function _onChangeRange() {
      logger.start("onChangeRange");
      this.enableOrDisableSelect();
    },
    onDateModeChange: function _onDateModeChange() {
      if (this.isSingleMode(this.modeSelect.getSelectedKey())) {
        this.setSingleMode();
      } else {
        this.setMultiMode();
      }
    },
    isSingleMode: function _isSingleMode(val) {
      val || (val = this.calendar.getSelectionMode());
      return val === CalendarSelectionMode.SINGLE;
    },
    setSingleMode: function _setSingleMode() {
      this.calendar.setSelectionMode(CalendarSelectionMode.SINGLE);
      this.calendar.unselectAllDates();
      this.selectButton.setVisible(false);
      this.clearButton.setVisible(false);
    },
    setMultiMode: function _setMultiMode() {
      this.calendar.setSelectionMode(CalendarSelectionMode.MULTIPLE);
      this.calendar.unselectAllDates();
      this.selectButton.setVisible(true);
      this.clearButton.setVisible(true);
      this.enableOrDisableSelect();
    },
    enableOrDisableSelect: function _enableOrDisableSelect() {
      const hasSelected = this.calendar.getSelectedDates().length > 0;
      this.selectButton.setEnabled(hasSelected);
      this.clearButton.setEnabled(hasSelected);
    },
    highlightExistingEntryDates: function _highlightExistingEntryDates() {
      if (this.entryDates) {
        this.clearExistingEntries();
      }

      const entries = this.getComponent().getModel(ModelNames.CurrentEntries).getEntries();
      const entryDates = entries.map(entry => new Date(entry.Date)); // clone the date to avoid mutation
      // slice to clone array, so entryDates Date objects are not changed to Strings

      this.calendar.toggleDatesType(entryDates.slice(0), Legend.Existing, true);
      this.entryDates = entryDates; // save them so we can revert later
    },
    clearExistingEntries: function _clearExistingEntries() {
      this.calendar.toggleDatesType(this.entryDates, Legend.Existing, false);
    },
    handleLoggedout: function _handleLoggedout() {
      this.clearExistingEntries();
    }
  });
  return CalendarController;
});