"use strict";

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

sap.ui.define(["bchui5/mvc/controller/AbstractController", "bchui5/util/BCHLogger", "bchui5/util/controls/FlashDialog", "bchui5/util/Device", "bchui5/util/EventBus", "com/bchydro/findmychargecode/mvc/model/ChargeCodes", "com/bchydro/findmychargecode/mvc/service/FavsService", "com/bchydro/findmychargecode/mvc/service/HttpService"], function (__AbstractController, __BCHLogger, __FlashDialog, Device, EventBus, __ChargeCodes, FavsService, HttpService) {
  function _interopRequireDefault(obj) {
    return obj && obj.__esModule && typeof obj.default !== "undefined" ? obj.default : obj;
  }

  var AbstractController = _interopRequireDefault(__AbstractController);

  var BCHLogger = _interopRequireDefault(__BCHLogger);

  var FlashDialog = _interopRequireDefault(__FlashDialog);

  var ChargeCodes = _interopRequireDefault(__ChargeCodes);

  /* global CBuffer */
  jQuery.sap.require("com.bchydro.findmychargecode.libraries.cbuffer");

  var NAME = "Details.controller";
  var logger = new BCHLogger(NAME);
  /**
   * @name com.bchydro.findmychargecode.mvc.controller.Details
   */

  var DetailsController = AbstractController.extend("com.bchydro.findmychargecode.mvc.controller.Details", {
    constructor: function _constructor() {
      AbstractController.prototype.constructor.call(this); // stores the charge code

      this.chargeCode = null;
    },
    _sameAsLastCode: false,
    cache: {
      cbuffer: null,
      children: {},
      parent: {}
    },
    onInit: function _onInit() {
      var _this = this;

      logger.start("onInit");
      this.details = this.byId("details");
      this.details.setBusyIndicatorDelay(0);
      this.parentList = this.byId("parentList");
      this.childrenList = this.byId("childrenList");
      this.favBtn = this.byId("favButton");
      this.page = this.byId("page");
      this.pageContent = this.byId("pageContent");
      this.i18n = this.getComponent().getModel('i18n');
      this.model = new ChargeCodes({});
      this.getView().setModel(this.model);
      this.getEventBus().subscribe("app", "logout", function () {
        _this.pageContent.setVisible(false); // it's hidden initially too

      });

      if (Device.isHybridOrLocalhost()) {
        // need a circular buffer on hybrid since the app stays open all the time.
        // get it on localhost too just for testing
        this.cache.cbuffer = new CBuffer(15);
      }
    },
    onBeforeFirstShow: function _onBeforeFirstShow() {
      logger.start("onBeforeFirstShow");
    },
    onBeforeShow: function _onBeforeShow(evt) {
      logger.start("onBeforeFirstShow", evt.direction);

      if (evt.direction !== "initial") {
        this._showContent();

        this.refreshView(evt);
      }
    },
    onAfterShow: function _onAfterShow(evt) {
      logger.start("onAfterShow", evt.direction); // initial is when the page gets loaded in a split app

      if (!this._sameAsLastCode && evt.direction !== "initial") {
        this._getParent();

        this._getChildren();
      }
    },
    refreshView: function _refreshView(evt) {
      logger.start("refreshView");
      var data = evt.data;
      var chargeCode = data.active;
      var oldChargeCode = this.chargeCode;

      if (oldChargeCode && oldChargeCode.ChargeCodeKey === chargeCode.ChargeCodeKey) {
        this._sameAsLastCode = true;

        this._flicker();

        return;
      }

      this.chargeCode = chargeCode;
      this._sameAsLastCode = false; // prepare the noDataText with the word 'Searching' before calling setData() which will clear the lists

      this._prepareParentText();

      this._prepareChildrenText();

      chargeCode.Parent = [];
      chargeCode.Children = [];
      var model = this.model;
      model.setData(chargeCode);
      model.refresh(); // this.getView().setModel(new JSONModel(this.chargeCode));

      this._toggleFavIcon(this.chargeCode.isFav);

      cycleCache(this.cache, chargeCode); // The actual load time is pretty much instantaneous, but we keep the busy
      // dialog because its hard to see that the information has changed (Just to make it clear to the user)

      this._flicker();
    },
    onNavBackPress: function _onNavBackPress() {
      logger.start("onNavBackPress");
      this.getRouter().navBack();
    },
    onListItemPress: function () {
      var _onListItemPress2 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee(evt) {
        var clickedItem, context, itemData;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                logger.start("onListItemPress");
                clickedItem = evt.getSource();
                context = clickedItem.getBindingContext();
                itemData = context.getObject();
                this.refreshView({
                  data: {
                    active: itemData
                  }
                });
                _context.next = 7;
                return Promise.all([this._getParent(), this._getChildren()]);

              case 7:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      return function _onListItemPress(_x) {
        return _onListItemPress2.apply(this, arguments);
      };
    }(),
    onFavIconPress: function _onFavIconPress() {
      logger.start("onFavIconPress");
      var chargeCode = this.chargeCode;
      var isFav = FavsService.toggle(chargeCode);
      var eventId = isFav ? "add" : "remove";
      EventBus.tryPublish(this.getEventBus(), "favs", eventId, chargeCode);

      this._toggleFavIcon(isFav);
    },
    _toggleFavIcon: function _toggleFavIcon(on) {
      logger.start("toggleFavIcon", on);
      this.favBtn.setIcon("sap-icon://".concat(on ? "favorite" : "unfavorite"));
    },
    _showContent: function _showContent() {
      logger.start("showContent");
      this.pageContent.setVisible(true); // it's hidden initially
    },
    _prepareParentText: function _prepareParentText() {
      logger.start("prepareParentText"); // searching will happen in onAfterShow, so set 'Searching' now for the transition

      if (this.chargeCode.hasParent) {
        this.parentList.setNoDataText(this.i18n.getProperty("searching"));
      }
    },
    _prepareChildrenText: function _prepareChildrenText() {
      logger.start("prepareChildrenText");

      if (ChargeCodes.isParentCandidate(this.chargeCode)) {
        this.childrenList.setNoDataText(this.i18n.getProperty("searching"));
      }
    },
    _resetParentText: function _resetParentText() {
      this.parentList.setNoDataText(this.i18n.getProperty("parent_no_data"));
    },
    _resetChildrenText: function _resetChildrenText() {
      this.childrenList.setNoDataText(this.i18n.getProperty("child_no_data"));
    },
    _flicker: function _flicker() {
      var details = this.details;
      details.setBusy(true);
      setTimeout(function () {
        details.setBusy(false);
      }, 50);
    },
    _getParent: function () {
      var _getParent2 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee2() {
        var thisList, chargeCode, thisCache, cacheKey, cachedResponse, searchCodeKey, searchCodeType, response;
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                logger.start("getParent");
                thisList = this.parentList;
                chargeCode = this.chargeCode;
                thisCache = this.cache.parent;
                cacheKey = getCacheKey(chargeCode);

                if (chargeCode.hasParent) {
                  _context2.next = 8;
                  break;
                }

                this._resetParentText();

                return _context2.abrupt("return");

              case 8:
                console.log("Checking parent cache for key:", cacheKey);
                cachedResponse = thisCache[cacheKey];

                if (!cachedResponse) {
                  _context2.next = 15;
                  break;
                }

                // debugger;
                this._resetParentText();

                console.log("Using cached parent for:", cachedResponse);
                this.model.setParentFromSearch(cachedResponse);
                return _context2.abrupt("return");

              case 15:
                thisList.setBusyIndicatorDelay(0);
                thisList.setNoDataText(this.i18n.getProperty("searching"));
                searchCodeKey = chargeCode.ChargeCodeKey;
                searchCodeType = chargeCode.ChargeCodeType;
                _context2.prev = 19;
                _context2.next = 22;
                return HttpService.get({
                  busy: thisList,
                  path: "ChargeCodeCollection(ChargeCodeType='" + searchCodeType + "',ChargeCodeKey='" + searchCodeKey + "')/Parent",
                  // this shouldShowError option is specific to this app's HttpService
                  shouldShowError: shouldShowErrorsCheck
                });

              case 22:
                response = _context2.sent;
                thisCache[cacheKey] = response;
                this.model.setParentFromSearch(response);
                _context2.next = 31;
                break;

              case 27:
                _context2.prev = 27;
                _context2.t0 = _context2["catch"](19);
                // HttpService will have shown the errors if needed.
                FlashDialog.unhandled(_context2.t0);
                this.model.setParent([]);

              case 31:
                _context2.prev = 31;

                // always revert it from the 'Searching' text
                this._resetParentText();

                return _context2.finish(31);

              case 34:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this, [[19, 27, 31, 34]]);
      }));

      return function _getParent() {
        return _getParent2.apply(this, arguments);
      };
    }(),
    _getChildren: function () {
      var _getChildren2 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee3() {
        var thisList, chargeCode, thisCache, cacheKey, cachedResponse, searchCodeKey, searchCodeType, response;
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                logger.start("getChildren");
                thisList = this.childrenList;
                chargeCode = this.chargeCode;
                thisCache = this.cache.children;
                cacheKey = getCacheKey(chargeCode);

                if (!ChargeCodes.isNotParentCandidate(chargeCode)) {
                  _context3.next = 8;
                  break;
                }

                this._resetChildrenText();

                return _context3.abrupt("return");

              case 8:
                console.log("Check child cache for cacheKey:", cacheKey);
                cachedResponse = thisCache[cacheKey];

                if (!cachedResponse) {
                  _context3.next = 15;
                  break;
                }

                // debugger;
                this._resetChildrenText();

                console.log("Cached children found:", cachedResponse);
                this.model.setChildrenFromSearch(cachedResponse); // cachedModel.refreshFavsProperty();
                // need to copy the model or else the list may not update
                // thisList.setModel(cachedModel.copy());

                return _context3.abrupt("return");

              case 15:
                thisList.setBusyIndicatorDelay(0);
                thisList.setNoDataText(this.i18n.getProperty("searching"));
                searchCodeKey = chargeCode.ChargeCodeKey;
                searchCodeType = chargeCode.ChargeCodeType;
                _context3.prev = 19;
                _context3.next = 22;
                return HttpService.get({
                  path: "ChargeCodeCollection(ChargeCodeType='" + searchCodeType + "',ChargeCodeKey='" + searchCodeKey + "')/Children",
                  busy: thisList,
                  // this shouldShowError option is specific to this app's HttpService
                  shouldShowError: shouldShowErrorsCheck
                });

              case 22:
                response = _context3.sent;
                thisCache[cacheKey] = response;
                this.model.setChildrenFromSearch(response);
                _context3.next = 31;
                break;

              case 27:
                _context3.prev = 27;
                _context3.t0 = _context3["catch"](19);
                FlashDialog.unhandled(_context3.t0);
                this.model.setChildren([]);

              case 31:
                _context3.prev = 31;

                this._resetChildrenText();

                return _context3.finish(31);

              case 34:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this, [[19, 27, 31, 34]]);
      }));

      return function _getChildren() {
        return _getChildren2.apply(this, arguments);
      };
    }()
  });

  function cycleCache(cache, chargeCode) {
    logger.start("cycleCache", chargeCode);
    var cbuffer = cache.cbuffer;

    if (!cbuffer) {
      return; // no circular buffer used on desktop
    }

    var cacheKey = getCacheKey(chargeCode); // TODO implement a touch method in the buffer that moves the key to the back
    // otherwise, a recently used key might get bumped

    var includesKey = cbuffer.includes(cacheKey);

    if (includesKey) {
      return;
    } // push the new key to the buffer and see if one got bumped


    var bumped = cbuffer.push(cacheKey);

    if (bumped) {
      // if a key has been bumped from the buffer,
      // also delete its saved search results
      delete cache.parent[bumped];
      delete cache.children[bumped];
    }
  }

  function getCacheKey(chargeCode) {
    return chargeCode.ChargeCodeKey + "-" + chargeCode.ChargeCodeType;
  }

  function shouldShowErrorsCheck(error) {
    if (error.status === 404) {
      logger.caught(error);
      return false; // suppress default handler/flash
    } else if (error.status === 500 && error.responseText.includes("This row is not contained in the table")) {
      logger.caught(error);
      return false; // suppress default handler/flash
    } else {
      // trigger default handler
      return true;
    }
  }

  return DetailsController;
});