sap.ui.define([], function () {
  /**
   * @returns {Promise|Null} Does not create a promise if callback mode is used.
   */
  function requireAsync(path, cb) {
    if (cb) {
      return asyncCallback(path, cb);
    }

    return new Promise((resolve, reject) => {
      try {
        //logger.start(`require: ${name}`)
        sap.ui.require([path], resolve);
      } catch (err) {
        window.console.error(err.stack);
        reject(err);
      }
    }).then(module => {
      if (typeof cb === "function") {
        try {
          cb(module);
        } catch (error) {
          window.console.error("Uncaught error in requireAsync callback", error);
        }
      }

      return module;
    });
  }

  function asyncCallback(path, cb) {
    try {
      sap.ui.require([path], theModule => {
        cb(theModule);
      });
    } catch (err) {
      window.console.error(err.stack);
    }
  }
  /**
  * Synchronous requiring is a 2 step process.
  * 1. first call jQuery.sap.require to load the module into UI5's modules.
  *    but it doesn't get returned.
  * 2. next use sync invocation of sap.ui.require (non-array).
  *    this would return null if it hasn't been loaded into UI5's modules yet.
  **/


  function requireSync(path) {
    if (sap.ui.requireSync) {
      return sap.ui.requireSync(path);
    } else {
      jQuery.sap.require(path);

      const module = sap.ui.require(path);

      if (!module) {
        throw new Error(`Unable to load module ${path}`);
      }

      return module;
    }
  }
  /**
   * Load JS or CSS Files via <script> or <link> tags.
   * Note. This can only be done in bootstrap js in a <script> tag itself.
   */


  function loadJs(filename, attrs) {
    return new Promise((resolve, reject) => {
      const fileref = document.createElement("script");
      fileref.setAttribute("type", "text/javascript");
      fileref.setAttribute("src", filename);

      if (typeof fileref !== "undefined") {
        //fileref.async = false;
        for (const k in attrs) {
          if (attrs.hasOwnProperty(k)) {
            fileref.setAttribute(k, attrs[k]);
          }
        } // fileref.onload = function () {


        fileref.onload = function () {
          resolve();
        };

        fileref.onerror = function (error) {
          console.error(`Failed to load ${filename}`);
          reject(error);
        };

        document.getElementsByTagName("html")[0].appendChild(fileref);
      }
    });
  }

  const Require = {
    requireSync,
    requireAsync,
    async: requireAsync,
    sync: requireSync,
    asyncCallback,
    js: loadJs
  };
  bchDbgAdd({
    Require
  });
  return Require;
});