/*!
 * (c) Copyright 2015 BC Hydro and Power Authority
 */
import SAPDialog from "sap/m/Dialog";
((window) => {
    // These all correspond to bchBtn<style> css styles
    window.bch.m.DialogStyle = {
        Success: "Success",
        Error: "Error",
        Danger: "Danger",
        Warning: "Warning",
        Info: "Info"
    };
})(window);
const supportedOptions = new Set([
    ...("beforeOpen,afterOpen,beforeClose,afterClose".split(",")),
    ...("classes,style,closeable,suppressOverflowFooter".split(",")),
    ...("title,content,customHeader,customFooter".split(","))
]);
/**
 * @name bch.m.Dialog
 */
export default class BCHDialog extends SAPDialog {
    constructor(id, settings) {
        const args = [];
        if (id) {
            args.push(id);
        }
        if (settings) {
            args.push(settings);
        }
        super(...args);
        const options = args.pop() || {};
        if (!options.noBCHClass) {
            this.addStyleClass("bchDialog");
        }
        if (options && options.classes) {
            this.addStyleClass(options.classes);
        }
        if (options && options.closeable) {
            this.addStyleClass("bchCloseableDialog");
        }
    }
    setStyle(style) {
        if (!style) {
            return this;
        }
        this.addStyleClass("bchDialog" + style);
        return this;
    }
    static open(...args) {
        const dialog = new BCHDialog(...args);
        dialog.open();
        return dialog;
    }
    async openAsync(...args) {
        this.open(...args); // synchronous
        return this; // resolve the async Promise
    }
    openAndAwaitClose(...args) {
        return new Promise((resolve, reject) => {
            try {
                this.attachEventOnce("afterClose", () => {
                    resolve(this);
                });
                this.open(...args);
            }
            catch (error) {
                reject(error);
            }
        });
    }
    close(cb) {
        return new Promise((resolve, reject) => {
            this.attachEventOnce("afterClose", () => {
                if (typeof cb === "function") {
                    window.console.assert(!cb, "Please use the Promise API of close");
                    try {
                        cb(this);
                    }
                    catch (error) {
                        return reject(error);
                    }
                }
                resolve(this);
            });
            super.close();
        });
    }
    /**
     * Close and Destroy.
     * Using this rather than only destroy ensures the beforeClose and afterClose
     * callbacks are called.
     * @param {function} [cb] optional after destroy callback. alternatively use the promise.
     * @return {Promise<BCHDialog>} this
     */
    async closeAndDestroy(cb) {
        if (this.isOpen()) {
            await this.close();
        }
        this.destroy(); // synchronous
        if (typeof cb === "function") {
            if (typeof (window).dbg === "object") {
                alert("Please use the Promise API of closeAndDestroy");
            }
            setTimeout(() => {
                cb(this);
            });
        }
        return this;
    }
    static copySupportedOptions(source, target = {}) {
        for (const [key, value] of Object.entries(source)) {
            if (supportedOptions.has(key)) {
                target[key] = value;
            }
        }
        return target;
    }
}
BCHDialog.renderer = {}; // inherit
BCHDialog.metadata = {
    properties: {
        classes: {
            type: "string",
            group: "Appearance",
            defaultValue: ""
        },
        style: {
            type: "bch.m.DialogStyle",
            group: "Appearance",
            defaultValue: "Info"
        },
        closeable: {
            type: "boolean",
            group: "Appearance",
            defaultValue: false
        },
        noBCHClass: {
            type: "boolean",
            group: "Appearance",
            defaultValue: false
        },
    },
};
// Not all SAP Dialog options are listed. Add as needed.
BCHDialog.supportedOptions = supportedOptions;
