//babel-sapuidefine-skip
/*
Bootstrap UI5 and the UI5 shell, using the app's Component.js.
Leave the rest to Component.js in order for FioriLaunchpad to work.

This file is NOT called when the app is loaded within FioriLaunchpad.

NOTE on ESNext:
Because this file loads before the babel-browser-polyfill is included,
be careful what features are used.
  const, let and arrow functions are safe since they are transpiled to ES5.
  Object.assign, Promises and async/await can only be used after the babel polyfill.
*/
; //eslint-disable-line no-extra-semi
(function () { //private scope

  const location = window.location
  const localStorage = window.localStorage

  // Get this script tag now before loadJsFile() is called.
  const thisScript = (function getThisSciptTag() {
    const scripts = document.getElementsByTagName('script')
    return scripts[scripts.length - 1]
  })()

  const isTest = !!thisScript.attributes.getNamedItem("data-bch-test")

  const bchUrl = (function getBootstrapRoot() {
    const mypath = thisScript.src.split('?')[0] // remove any ?query
    return mypath.split('/').slice(0, -1).join('/') // remove last filename part of path
  })()

  const bchPath = bchUrl.replace(window.origin, '')

  /**
   * Add a <title> tag to the <head>
   */
  function addPageTitle(title) {
    const titleElement = document.createElement("title")
    titleElement.textContent = title
    document.getElementsByTagName("head")[0].appendChild(titleElement)
  }

  /*
   * Gets the context path (i.e. /AppName/page/unit/ will result in /AppName/page)
   */
  function getContextPath() {
    return location.pathname.split("/").slice(0, -1).join("/")
  }

  /**
   * Asynchronously load some json
   */
  function loadJSON(filename, callback) {
    const xhr = new window.XMLHttpRequest()
    xhr.open("GET", filename, true)
    xhr.responseType = "json"
    xhr.onreadystatechange = function () {
      if (xhr.readyState === 4) { // DONE
        if (xhr.status > 299) {
          let responseText
          try {
            //responseText is a getter and may throw an InvalidStateError in some cases.
            responseText = xhr.responseText
          }
          catch (ignore) {
            responseText = ""
          }
          alert(`Unable to load ${filename}\n${xhr.status}-${xhr.statusText}\n${responseText}`)
          return
        }
        const response = xhr.response
        if (typeof response === "string") {
          callback(JSON.parse(response))
        }
        else {
          callback(response)
        }
      }
    }
    xhr.send()
  }

  function getAppConfig(callback) {
    const contextPath = isFile() ? "" : (getContextPath() + "/")
    loadJSON(contextPath + "config/app.json", callback)
  }

  function isLocal() {
    return /^(localhost|myserver)$/.test(location.hostname)
  }

  function isBeta() {
    return /beta/.test(location.search)
  }

  function isFile() {
    return /file/.test(location.protocol)
  }

  function hasDebugQuery(location) {
    return /debug/.test(location.search)
  }

  function hasUI5DebugQuery() {
    return /ui5debug|sap-ui-debug/.test(window.location.search)
  }

  /**
   * Get the runtime.
   */
  function getRuntime() {
    return (
      isFile() ? "native" :
      !isLocal() ? "server" :
      isBeta() ? "beta" : "local"
    )
  }

  /**
   * Helper method to read a appConfig value from app.json.
   * Value can be a simple value, or an object in the format:
   * runtime.key has a higher priority than key.server.
   *
   * key: {
   *   beta: "value",
   *   local: "value",
   *   server: "serverValue2",
   *   native: "value",
   *   default: "value"
   * },
   * server: {
   *   key: "serverValue1"
   * }
   */
  function getEnvironmentConfigParam(appConfig, key, deflt) {
    const runtime = getRuntime()
    const runtimeConf = appConfig[runtime] || {}

    //always check runtime.key first, then just key
    let value = runtimeConf[key]
    if (!value && value !== false) {
      value = appConfig[key]
    }

    //assume the props we're dealing with here are primitives
    if (typeof value === "object") {
      const obj = value
      value = obj[runtime] || obj.default
    }
    return value || deflt
  }

  /**
   * Stores or removes the UI5 debug flag.
   * Needs to be done before UI5 gets bootstraped.
   * Especially removing the flag if the -dbg.js files are not available.
   */
  function setSAPUIDebugProperty(debug) {
    if (debug === "false") {
      debug = false
    }
    if (debug) {
      localStorage.setItem("sap-ui-debug", "X")
    }
    else {
      localStorage.removeItem("sap-ui-debug")
    }
  }

  function useUI5Debug(appConfig) {
    //for hybrid, use appConfig debug value, else use query string
    return isFile() ? appConfig["sap-ui-debug"] : hasUI5DebugQuery()
  }

  function getUI5CorePath(appConfig, ui5BasePath) {
    const ui5DebugOn = useUI5Debug(appConfig)
    setSAPUIDebugProperty(ui5DebugOn)
    return ui5BasePath + "sap-ui-core" + (ui5DebugOn ? "-dbg" : "") + ".js"
  }

  function getUI5BasePath(appConfig) {
    //read ui5_path from app.json. it may be a string
    const newAppConfig = isTest ? {...appConfig, ...appConfig.test} : appConfig
    let ui5Resources = getEnvironmentConfigParam(newAppConfig, "ui5_path", "resources")
    if (ui5Resources.length && ui5Resources.slice(-1) !== "/") {
      ui5Resources += "/"
    }

    //handle resources in app.json path to support gateway cachebuster url
    if (ui5Resources.indexOf("resources") === -1) {
      ui5Resources += "resources/"
    }
    return ui5Resources
  }

  function useBchDebug(appConfig) {
    const appConfigDebug = appConfig.debug
    if (isFile()) {
      //for mobile, only use the app.json debug value
      return appConfigDebug
    }
    else if (isLocal()) {
      //for localhost, the default is on. only disable if either flag is off/false
      const overrideOff = /nodebug|(debug=off)/.test(location.search)
      return (appConfigDebug !== false) && !overrideOff
    }
    else {
      //for desktop deployment, only check the query string. the default is off so only look for the word debug to enable
      return hasDebugQuery(window.location)
    }
  }

  function bchDebugLogic(appConfig) {
    const dbg = useBchDebug(appConfig) && {}
    if (dbg) {
      window.dbg = dbg
    }

    // Always add these dbg methods to window. The flag will internally decide what to do.
    Object.assign(window, {
      bchDbg: dbg, // could be a dynamic getter prop later if needed to override
      bchDbgAdd(obj) {
        if (dbg) {
          Object.keys(obj).forEach((key) => {
            dbg[key] = obj[key]
          })
        }
      },
      bchDbgAlert(...msgs) {
        if (dbg) {
          alert(msgs.join(" "))
        }
      },
      bchDbgWarn(...msgs) {
        if (dbg) {
          console.warn(...msgs)
        }
      }
    })
  }

  function loadCssFiles(cssArray) {
    const head = document.getElementsByTagName('head')[0]
    for (let i = 0; i < cssArray.length; i++) {
      const linkref = document.createElement("link")
      const config = cssArray[i]
      linkref.setAttribute("rel", "stylesheet")
      linkref.setAttribute("type", "text/css")
      linkref.setAttribute("charset", "utf-8")
      linkref.setAttribute("media", "screen")
      linkref.setAttribute("href", config.href)
      if (config.id) {
        linkref.setAttribute("id", config.id)
      }
      head.appendChild(linkref)
    }
  }

  /**
   * Load JS Files via <script> tags.
   * Note. This can only be done in bootstrap js in a <script> tag itself.
   */
  function loadJsFile(filename, attrs, callback, noAlert) {
    const fileref = document.createElement("script")
    let called = false //to prevent double callback call
    fileref.setAttribute("type", "text/javascript")
    fileref.setAttribute("src", filename)
    if (typeof fileref !== "undefined") {
      for (const k in attrs) {
        if (attrs.hasOwnProperty(k)) {
          fileref.setAttribute(k, attrs[k])
        }
      }
      fileref.onload = fileref.onreadystatechange = function () {
        if (!called && (!this.readyState || this.readyState === 'complete')) {
          called = true
          callback && callback()
        }
      }
      fileref.onerror = function () {
        if (!noAlert) {
          alert("Failed to load " + filename)
        }
      }
      document.getElementsByTagName("html")[0].appendChild(fileref)
    }
  }

  function getCore() {
    try {
      return sap.ui.getCore()
    }
    catch (err) {
      alert(`Error loading UI5 Core. Please notify support\n${err.message}`)
    }
  }

  function initShell(appConfig) {
    getCore().attachInit(() => {
      const appModulePath = appConfig.appModule || appConfig.appId.toLowerCase()
      const contextPath = location.protocol === "file:" ? "./" : (getContextPath() + "/")
      jQuery.sap.registerModulePath(appModulePath, contextPath)

      //in case there is no content div. mostly for qunits, where qunit.js may remove it
      //TODO remove this in favor of noshell mode for qunits
      if (!document.getElementById('content')) {
        jQuery("body").append("<div id='content'>")
      }

      // const ComponentContainer = requireSync("sap/ui/core/ComponentContainer")
      // const Shell = requireSync("sap/m/Shell")
      sap.ui.require([
        "sap/ui/core/ComponentContainer",
        "sap/m/Shell"
      ], (ComponentContainer, Shell) => {
        // loadUI5LibraryParameters()
        if (!ComponentContainer && isLocal()) {
          window.console.assert(ComponentContainer, "ComponentContainer is null. Disable devtools cache or disable mobile emulation or add sap-ui-xx-fakeOS=iphone url parameter")
        }
        const shell = new Shell({
          appWidthLimited: false, // if `false`, the app container will be 100% wide
          app: new ComponentContainer({
            height: "100%",
            name: appModulePath
          })
        })
        shell.addStyleClass("bchShell")
        shell.placeAt("content")
        dispatchBootstrappedEvent()
      })
    })
  }

  /**
   * Merge app specific ui5 appConfig with the defaults
   * Used in UI5 bootstrapping.
   */
  function getUi5Config(appConfig, ui5Theme, ui5Libs) {

    const libs = [...ui5Libs, "bchui5"]

    const config = {
      "id": "sap-ui-bootstrap",
      "data-sap-ui-resourceroots": `{"bchui5" : "${bchPath}"}`,
      "data-sap-ui-libs": libs.join(", "),
      // "data-sap-ui-libs": "sap.m",
      "data-sap-ui-xx-bindingSyntax": "complex",
      "data-sap-ui-theme": ui5Theme,
      "data-sap-ui-preload": "async",
      "data-sap-ui-theme-bchui5": "false",
      "data-sap-ui-theme-bch.m": "false",
      "data-sap-ui-language": "en",
      ...appConfig.ui5
    }

    //<script>window["sap-ui-config"]={preloadLibCss:["!sap.ui.core","sap.m","sap.ui.layout"]}</script>

    // Copy the sap-ui config from the script tag.
    // NOTE: attributes is not iterable so don't use forEach or for-of
    const attrs = thisScript.attributes
    for (let i = 0; i < attrs.length; i++) {
      const attr = attrs.item(i)
      if (attr.name.startsWith("data-sap")) {
        config[attr.name] = attr.value
      }
    }
    return config
  }

  function getUI5Libs(appConfig) {
    const defaultLibs = ["sap.ui.core", "sap.m", "sap.ui.layout"]
    return [
      ...(appConfig.libs || defaultLibs),
      ...(appConfig.extraUI5Libs || []) //i.e. ["sap.uxap"]
    ]
  }

  function getUI5Theme(appConfig) {
    const theme = appConfig.theme || "sap_bluecrystal"
    return theme.indexOf("sap_") === 0 ? theme : `sap_${theme}`
  }

  function dispatchBootstrappedEvent() {
    if (isTest) {
      try {
        document.dispatchEvent(new Event('bch-bootstrapped')) //event used in unit tests
      }
      catch (ignore) {
        //Fails on IE11 and older.
      }
    }
  }

  function loadUI5Css(ui5BasePath, ui5Theme, ui5Libs) {
    const css = ui5Libs.map(lib => ({
      href: `${ui5BasePath}${lib.replace(/\./g, '/')}/themes/${ui5Theme}/library.css`,
      id: `sap-ui-theme-${lib}`
    }))
    loadCssFiles(css)
  }

  function loadOurCss(appConfig) {
    const qs = `${window.dbg ? `?_=${Date.now()}` : ''}`
    const css = [{
      href: `${bchPath}/css/bch-style.css${qs}`
    }]
    if (appConfig.bootstrapAppCSS) {
      css.push({
        href: `css/app-style.css${qs}`
      })
    }
    loadCssFiles(css)
  }

  function loadUI5(appConfig, callback) {
    const ui5BasePath = getUI5BasePath(appConfig)
    const ui5CorePath = getUI5CorePath(appConfig, ui5BasePath)
    const ui5Libs = getUI5Libs(appConfig)
    const ui5Theme = getUI5Theme(appConfig)
    if (appConfig.bootstrapCSS) {
      loadUI5Css(ui5BasePath, ui5Theme, ui5Libs)
      loadOurCss(appConfig) // This is not ideal inside loadUI5 method, but it needs to happen between UI5 CSS and UI5 JSv
    }
    const ui5Config = getUi5Config(appConfig, ui5Theme, ui5Libs)
    console.log(`Loading ui5Core: ${ui5CorePath}`)
    loadJsFile(ui5CorePath, ui5Config, callback)
  }

  function loadBluebird(callback) {
    const fullPath = `${bchUrl}/libraries/bluebird${bchDbg ? '.js' : '-min.js'}`
    loadJsFile(fullPath, {}, callback)
  }

  function loadBabelPolyfill(callback) {
    const fullPath = `${bchUrl}/libraries/babel-polyfill-min.js`
    loadJsFile(fullPath, {}, callback)
  }

  function loadAbortController(callback) {
    const fullPath = `${bchUrl}/libraries/abortcontroller.js`
    loadJsFile(fullPath, {}, callback)
  }

  function addPromiseFinally(PromiseClass) {
    PromiseClass.prototype['finally'] = function finallyPolyfill(callback) {
      const constructor = this.constructor
      return this.then(
        (value) => constructor.resolve(callback()).then(() => value),
        (reason) => constructor.resolve(callback()).then(() => { throw reason })
      )
    }
  }

  function addPromiseFinallyToNativePromise() {
    if (window.Promise) {
      //window.NativePromise = window.Promise
      // If there is a native Promise, extend it now with the finally method, before loading bluebird.
      // Although window.Promise will use the bluebird one, some native functions will return the native Promise.
      // Particularly async/await functions in Chrome 55.
      addPromiseFinally(window.Promise)
    }
  }

  /*
   * Init. Called on load.
   */
  function init() {
    if (isLocal()) {
      loadJsFile("//localhost:35729/livereload.js", {}, null, true)
    }
    addPromiseFinallyToNativePromise()
    // Load app config first so we can load in Piwik and also determine if we're in debug mode
    getAppConfig((appConfig) => {
      bchDebugLogic(appConfig)
      addPageTitle(appConfig.name)
      loadBabelPolyfill(() => {
        loadBluebird(() => {
          loadAbortController(() => {})

          addPromiseFinally(window.Promise) //Now add finally to the Bluebird promise which is now in window.Promise.
          loadUI5(appConfig, () => {
            //jQuery.sap.log.setLevel(jQuery.sap.log.Level.DEBUG)
            jQuery("html").addClass("bch")
            if (isTest && !!thisScript.attributes.getNamedItem("data-bch-noshell")) {
              return dispatchBootstrappedEvent()
            }
            else {
              // Note that there is no dispatchBootstrapped() event here.
              // The BCH Component will dispatch its own event when done loading.
              return initShell(appConfig)
            }
          })
        })
      })
    })
  }

  init()

}())
